package net.northking.cctp.upperComputer.driver.ios;


import net.northking.cctp.upperComputer.driver.ios.command.data.AppBatteryData;
import net.northking.cctp.upperComputer.driver.ios.command.data.ScreenInfoData;
import net.northking.cctp.upperComputer.driver.ios.command.data.SearchUiNodeData;
import net.northking.cctp.upperComputer.driver.ios.command.data.UiNodeData;
import net.northking.cctp.upperComputer.driver.ios.command.listener.AppBatteryDataListener;
import net.northking.cctp.upperComputer.driver.ios.command.listener.ScreenInfoDataListener;
import net.northking.cctp.upperComputer.driver.ios.packet.*;
import net.northking.cctp.upperComputer.driver.usbmuxd.AppleDevice;
import net.northking.cctp.upperComputer.driver.usbmuxd.UsbMuxd;

import java.io.IOException;
import java.util.ArrayList;

public final class NKAgent {

    public static final int PORT = 50001;

    private final DeviceSocketProvider deviceSocketProvider;

    private PacketTransfer packetTransfer;

    private final ArrayList<OnAppBatteryDataListener> onAppBatteryDataListeners = new ArrayList<>();

    private final ArrayList<OnScreenInfoDataListener> onScreenInfoDataListeners = new ArrayList<>();

    private Runnable onDisconnectListener = null;

    public NKAgent(AppleDevice appleDevice) {
        this(new UsbMuxd(), appleDevice);
    }

    public NKAgent(UsbMuxd usbMuxd, AppleDevice appleDevice) {
        this(new LocalUsbmuxdDeviceSocketProvider(usbMuxd, appleDevice, PORT));
    }
    public NKAgent(DeviceSocketProvider deviceSocketProvider) {
        this.deviceSocketProvider = deviceSocketProvider;
    }

    public String getUUID() {
        return deviceSocketProvider.getUdid();
    }

    public void connect() throws IOException {
        System.out.println("NKAgent connect");
        initTransfer();
        initAsyncListener();
    }


    public void setOnDisconnectListener(Runnable listener) {
        this.onDisconnectListener = listener;
        if (packetTransfer != null) {
            packetTransfer.setOnDisconnectListener(listener);
        }
    }

    private void initTransfer() throws IOException {
        DeviceSocketStream deviceSocketStream = deviceSocketProvider.createConnect();
        packetTransfer = new PacketTransfer(deviceSocketStream.getInputStream(), deviceSocketStream.getOutputStream(), deviceSocketProvider.getUdid());
        if (this.onDisconnectListener != null) {
            packetTransfer.setOnDisconnectListener(this.onDisconnectListener);
        }
    }

    private void initAsyncListener() {
        if (packetTransfer == null) throw new RuntimeException("先调用initTransfer再初始化异步监听");
        packetTransfer.addAsyncListener(new AppBatteryDataListener() {
            @Override
            public void handle(PacketTransfer transfer, ICommandPacket packet) {
                super.handle(transfer, packet);
                ArrayList<OnAppBatteryDataListener> listeners = new ArrayList<OnAppBatteryDataListener>(onAppBatteryDataListeners.size());
                synchronized (onAppBatteryDataListeners) {
                    listeners.addAll(onAppBatteryDataListeners);
                }
                for (OnAppBatteryDataListener l : listeners) {
                    l.onAppBatteryData(getAppBatteryData());
                }
            }
        });
        packetTransfer.addAsyncListener(new ScreenInfoDataListener() {
            @Override
            public void handle(PacketTransfer transfer, ICommandPacket packet) {
                super.handle(transfer, packet);
                ArrayList<OnScreenInfoDataListener> listeners = new ArrayList<OnScreenInfoDataListener>(onScreenInfoDataListeners.size());
                synchronized (onScreenInfoDataListeners) {
                    listeners.addAll(onScreenInfoDataListeners);
                }
                for (OnScreenInfoDataListener l : listeners) {
                    l.onScreenInfoData(getScreenInfoData());
                }
            }
        });
    }

    public void addOnAppBatteryDataListener(OnAppBatteryDataListener listener) {
        synchronized (onAppBatteryDataListeners) {
            onAppBatteryDataListeners.add(listener);
        }
    }

    public void removeOnAppBatteryDataListener(OnAppBatteryDataListener listener) {
        synchronized (onAppBatteryDataListeners) {
            onAppBatteryDataListeners.remove(listener);
        }
    }

    public void addOnScreenInfoDataListener(OnScreenInfoDataListener listener) {
        synchronized (onScreenInfoDataListeners) {
            onScreenInfoDataListeners.add(listener);
        }
    }

    public void removeOnScreenInfoDataListener(OnScreenInfoDataListener listener) {
        synchronized (onScreenInfoDataListeners) {
            onScreenInfoDataListeners.remove(listener);
        }
    }

    public ScreenInfoData screenInfo() {
        ICommandPacket packet = packetTransfer.syncSend(2);
        if (packet == null) return null;
        ScreenInfoData screenInfo = new ScreenInfoData();
        packet.fillICommandData(screenInfo);
        return screenInfo;
    }

    public TextCommandData uiTreeJson() {
        ICommandPacket packet = packetTransfer.syncSend(3);
        if (packet == null) return null;
        TextCommandData stringCommandData = new TextCommandData();
        packet.fillICommandData(stringCommandData);
        return stringCommandData;
    }

    public UiNodeData uiTreeNode() {
        ICommandPacket packet = packetTransfer.syncSend(4);
        if (packet == null) return null;
        UiNodeData node = new UiNodeData();
        packet.fillICommandData(node);
        return node;
    }

    public UiNodeData uiNodeInfo(SearchUiNodeData searchUiNodeData) {
        ICommandPacket packet = packetTransfer.syncSend(26, searchUiNodeData);
        if (packet == null) return null;
        UiNodeData node = new UiNodeData();
        packet.fillICommandData(node);
        return node;
    }

    /**
     * 设置屏幕方向
     * @param aByte 取值范围：[1, 4]
     * @return 是否设置成功
     */
    public boolean setScreenOrientation(Byte aByte) {
        if (aByte < 1 || aByte > 4) {
            throw new IllegalArgumentException("参数范围错误");
        }
        ByteCommandData data = new ByteCommandData();
        data.aByte = aByte;
        ICommandPacket packet = packetTransfer.syncSend(3, data);
        BooleanCommandData bool = new BooleanCommandData();
        packet.fillICommandData(bool);
        return bool.isSuccess();
    }

    public boolean clickUiNode(SearchUiNodeData searchUiNodeData,int commandId) {
        ICommandPacket packet = packetTransfer.syncSend(commandId, searchUiNodeData);
        if (packet == null) return false;
        BooleanCommandData bool = new BooleanCommandData();
        packet.fillICommandData(bool);
        return bool.isSuccess();
    }

    public boolean handleTextUiNode(ICommandData data, int commandId) {
        ICommandPacket packet = packetTransfer.syncSend(commandId, data);
        if (packet == null) return false;
        BooleanCommandData bool = new BooleanCommandData();
        packet.fillICommandData(bool);
        return bool.isSuccess();
    }

    public boolean handleDeleteTextUiNode(int commandId) {
        ICommandPacket packet = packetTransfer.syncSend(commandId);
        if (packet == null) return false;
        BooleanCommandData bool = new BooleanCommandData();
        packet.fillICommandData(bool);
        return bool.isSuccess();
    }

    /*
   设置JPEGServer的帧率
   rate 取值[0, 15]
    */
    public boolean setMJPEGServerRate(int rate) {
        if (rate < 0 || rate > 60) {
            throw new IllegalArgumentException("参数范围错误");
        }
        IntCommandData data = new IntCommandData();
        data.anInt = rate;
        ICommandPacket packet = packetTransfer.syncSend(28, data);
        if (packet == null) return false;
        BooleanCommandData bool = new BooleanCommandData();
        packet.fillICommandData(bool);
        return bool.isSuccess();
    }
//
//    public boolean clickUiNode(SearchUiNodeData.SearchChain chain) {
//        return clickUiNode(chain.toSearchString(null));
//    }
//
//    public boolean clickUiNode(String searchChainText) {
//        var sendData = new TextCommandData();
//        sendData.setText(searchChainText);
//        var packet = packetTransfer.syncSend(6, sendData);
//        var bool = new BooleanCommandData();
//        packet.fillICommandData(bool);
//        return bool.isSuccess();
//    }

    public interface OnAppBatteryDataListener {
        void onAppBatteryData(AppBatteryData appBatteryData);
    }

    public interface OnScreenInfoDataListener {
        void onScreenInfoData(ScreenInfoData screenInfoData);
    }


}
